# -*- coding: utf-8 -*-
import json
from Crypto.Hash import SHA256

from django.template.loader import render_to_string
from django.utils.importlib import import_module
from django.conf import settings

from . import models
from . import settings as app_settings
from . import exceptions


def _import_func(func_addr):
    if not '.' in func_addr:
        raise ValueError(
            "'%s' is not a func address" % func_addr
        )

    try:
        module = import_module('.'.join(
            func_addr.split('.')[:-1]
        ))
    except ImportError:
        raise ValueError('Can`t find function %s' % func_addr)
    try:
        func = getattr(module, func_addr.split('.')[-1])
    except AttributeError:
        raise ValueError('Can`t find function %s' % func_addr)
    return func


def get_action_func(func_addr):
    try:
        func = _import_func(func_addr)
    except ValueError as e:
        raise exceptions.NotActionException(str(e))

    if not hasattr(func, 'confirm_action') or not func.confirm_action:
        raise exceptions.NotActionException(
            "%s not an action" % func_addr
        )

    return func


def set_action(user_contact, func_addr, func_kwargs=None, message_template=None,
               template_context=None, live_time=None,
               generate_code_func=None, send_code_func=None):
    """
    Sets an action. Checks that func_addr - is string with address of function
    and this function is confirm_method action else
    raises :exception:`confirmaction.exceptions.NotActionException`
    checks that all func_kwargs keys, and values are string. else raises
    `confirmaction.exceptions.WrongParamsException`.
    Sets live time from live_time param or settings.LIVE_TIME_CODE,
    Generates_code from generate_code_func or from settings.CONFIRM_GENERATION
    Generates message with message_template or with settings.CONFIRM_MESSAGE_TEMPLATE and
    template_context with action or {}
    Sends code to user_contact via send_code_func or from settings.CONFIRM_SEND_METHOD
    :param user_contact: user contact for sending activation code
    :type user_contact: string
    :param func_addr: address of action function
    :type func_addr: basestring
    :param func_kwargs: - dictionary of params. Keys and values are strings
    :type func_kwargs: - dict
    :param live_time: live time of action could be None
    :type live_time: int
    :param generate_code_func: function for generation code could be None
    :type generate_code_func: function
    :param send_code_func: function for sending code could be None
    :type send_code_func: function
    :returns: action object
    :rtype: :class:`confirmaction.models.Action`
    """
    # TODO: return integer not action
    # TODO: add encription  for code and identifier

    get_action_func(func_addr)

    if func_kwargs is None:
        func_kwargs = {}

    if generate_code_func is None:
        generate_code_func = _import_func(app_settings.CONFIRM_GENERATION)

    code = generate_code_func()

    if send_code_func is None:
        send_code_func = _import_func(app_settings.CONFIRM_SEND_METHOD)

    message_template = (
        message_template or app_settings.ACTIVATION_MESSAGE_TEMPLATE
    )

    template_context = template_context or {}
    template_context.update({'code': code })

    try:
        send_code_func(
            user_contact,
            render_to_string(message_template, template_context)
        )
    except Exception as e:
        raise exceptions.DidNotSendMessage(str(e))

    action = models.Action.objects.create(
        user_contact=user_contact,
        action_func=func_addr,
        confirm_code=SHA256.new(settings.SECRET_KEY + code).hexdigest(),
        live_time=live_time or app_settings.CODE_LIVE_TIME
    )

    models.ActionArg.objects.bulk_create(
        [
            models.ActionArg(action=action, name=str(key), value=str(value))
            for key, value in func_kwargs.iteritems()
        ]
    )
    return action


def apply_action(action_pk, code):
    """
    Applies action if code is correct and not out of date and is opened
    If it is out of date raise OutOfDate exception.
    If code is wrong raise WrongCode exception
    If code isn't opened raise UsedAction
    If all ok return status confirmaction.models.Action.FINISHED_SUCCESS and
    dict with result
    If something goes wrong return error status and message of error
    """
    try:
        action = models.Action.objects.get(pk=action_pk)
    except models.Action.DoesNotExist:
        raise exceptions.CantFindAction(action_pk)

    if not action.is_actual():
        raise exceptions.OutOfDate

    if not action.confirm_code == code:
        raise exceptions.WrongCode

    if action.action_status != models.Action.NOT_CONFIRMED:
        raise exceptions.UsedAction

    func = get_action_func(action.action_func)
    try:
        data = func(**action.get_kwargs())
    except exceptions.ErrorDuringProcess as e:
        action.action_status = models.Action.ERROR_DURING_PROCESS
        data = {'error': action.get_action_status_display()}
    except Exception as e:
        action.action_status = models.Action.ACTION_FAULT
        data = {'error': action.get_action_status_display()}
    else:
        if not isinstance(data, dict):
            action.action_status = models.Action.WRONG_RETURNED_DATA
            data = {'error': action.get_action_status_display()}
        else:
            action.action_status = models.Action.FINISHED_SUCCESS
    finally:
        action.result = json.dumps(data)
        action.save()

    return action.action_status, data




# -*- coding: utf-8 -*-
from django import test

from ..decorators import confirm_action


class TestSettingDecorator(test.TestCase):

    def test_set_decorator(self):

        @confirm_action
        def check_function():
            return

        self.assertTrue(hasattr(check_function, 'confirm_action'))
        self.assertTrue(check_function)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Action'
        db.create_table(u'confirmaction_action', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')()),
            ('action_func', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('confirm_code', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('updated', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal(u'confirmaction', ['Action'])

        # Adding model 'ActionArg'
        db.create_table(u'confirmaction_actionarg', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('action', self.gf('django.db.models.fields.related.ForeignKey')(related_name='arg_set', to=orm['confirmaction.Action'])),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('value', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal(u'confirmaction', ['ActionArg'])


    def backwards(self, orm):
        # Deleting model 'Action'
        db.delete_table(u'confirmaction_action')

        # Deleting model 'ActionArg'
        db.delete_table(u'confirmaction_actionarg')


    models = {
        u'confirmaction.action': {
            'Meta': {'object_name': 'Action'},
            'action_func': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'confirm_code': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'created': ('django.db.models.fields.DateTimeField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        u'confirmaction.actionarg': {
            'Meta': {'object_name': 'ActionArg'},
            'action': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'arg_set'", 'to': u"orm['confirmaction.Action']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        }
    }

    complete_apps = ['confirmaction']


from django.contrib import admin

from . import models


class ActionAdmin(admin.ModelAdmin):
    list_display = ('user_contact', 'action_func', 'get_kwargs',
                    'get_action_status_display',
                    'created', 'executed', 'result', 'live_time')
    search_fields = ('user_contact', 'action_func')
    list_filter = ('action_status', )
admin.site.register(models.Action, ActionAdmin)



# -*- coding: utf-8 -*-


class NotActionException(Exception):
    """
    Error raises when passed function not an action.
    """
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value


class CantFindAction(Exception):
    """
    Error raises when can't find action with specia identifier
    """
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return "Cant find action with identifier %d " % self.value


class WrongDataReturn(Exception):
    """
    Error raises when return data of function not dict
    """
    def __str__(self):
        return "Action should return only dict"


class OutOfDate(Exception):
    """
    Raise exception when tries to start old action
    """
    def __str__(self):
        return "Action out of date"


class WrongCode(Exception):
    """
    Raises exception when somebody used wrong code
    """
    def __str__(self):
        return "Wrong code"


class UsedAction(Exception):
    """
    Raises exception when somebody tries to apply wrong code
    """


class ErrorDuringProcess(Exception):
    # TODO: add value
    """
    Error raised in action if something goes wrong in action.

    Example:
    try:
        User.objects.get(pk=12)
    except User.DoesNotExist:
        raise ErrorDuringProcess("There is no user with pk 12")
    """


class DidNotSendMessage(Exception):
    """
    Error raises when passed function not an action.
    """
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value




# -*- coding: utf-8 -*-
import datetime

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone

from . import settings as app_settings


class Action(models.Model):
    """
    Model for action
    """
    OUT_OF_DATE = -1
    OPENED = 0
    CONFIRMED = 1

    CODE_STATUSES = (
        (OUT_OF_DATE, _("Out of date")),
        (OPENED, _("Opened")),
        (CONFIRMED, _("Confirmed"))
    )

    NOT_CONFIRMED = 0
    FINISHED_SUCCESS = 1
    WRONG_RETURNED_DATA = -1
    ERROR_DURING_PROCESS = -2
    ACTION_FAULT = -3

    ACTION_STATUS = (
        (NOT_CONFIRMED, _("Not confirmed")),
        (FINISHED_SUCCESS, _("Finished success")),
        (WRONG_RETURNED_DATA, _("Wrong returned data")),
        (ERROR_DURING_PROCESS, _('Error during process')),
        (ACTION_FAULT, _('Action fault'))
    )

    user_contact = models.CharField(
        verbose_name=_("User contact"),
        max_length=255
    )
    action_func = models.CharField(
        verbose_name=_("Action function"),
        max_length=255,
    )
    confirm_code = models.CharField(
        verbose_name=_("Confirm code"),
        max_length=255
    )
    created = models.DateTimeField(
        verbose_name=_("Created"),
        auto_now_add=True
    )
    updated = models.DateTimeField(
        verbose_name=_("Updated"),
        auto_now=True
    )
    executed = models.DateTimeField(
        verbose_name=_("Executed"),
        blank=True,
        null=True
    )
    action_status = models.IntegerField(
        verbose_name=_("Action status"),
        choices=ACTION_STATUS,
        default=NOT_CONFIRMED
    )
    result = models.TextField(
        verbose_name=_("Result"),
        blank=True,
        null=True
    )
    live_time = models.IntegerField(
        verbose_name=_("Live time"),
        blank=True,
        null=True
    )

    def is_actual(self):
        return timezone.now() < self.created + datetime.timedelta(
            minutes=self.live_time or app_settings.CODE_LIVE_TIME
        )

    def get_kwargs(self):
        return {x.name:x.value for x in self.arg_set.all()}


class ActionArg(models.Model):
    """
    Model for saving params of action
    """
    action = models.ForeignKey(
        'Action',
        related_name='arg_set'
    )
    name = models.CharField(
        _("Argument name"),
        max_length=255
    )
    value = models.CharField(
        _("Argument value"),
        max_length=255
    )

    class Meta:
        verbose_name = _("Action argument")
        verbose_name_plural = _("Action arguments")


# -*- coding: utf-8 -*-
import logging
import random

from django.conf import settings

logger = logging.getLogger('django')


def generate_code():
    """
    Default function for generating confirmation code
    """
    return ''.join(str(random.randint(0,9)) for x in range(4))


def send_code(user_contact, message):
    """
    Sends message to user. Works only in debug mode. In production mode should
    be implemented
    :param user_contact: email/phone etc for sending message to user
    :type user_contact: basestring
    :param message: message with activation code for user
    :type message: basestring
    """
    logger.critical(
        "Should be implemented in production and set in CONFIRM_SEND_METHOD"
    )
    logger.debug("Contact: %s" % user_contact)
    logger.debug(message)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Action.executed'
        db.add_column(u'confirmaction_action', 'executed',
                      self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True),
                      keep_default=False)

        # Adding field 'Action.action_status'
        db.add_column(u'confirmaction_action', 'action_status',
                      self.gf('django.db.models.fields.IntegerField')(default=0),
                      keep_default=False)

        # Adding field 'Action.result'
        db.add_column(u'confirmaction_action', 'result',
                      self.gf('django.db.models.fields.TextField')(null=True, blank=True),
                      keep_default=False)

        # Adding field 'Action.live_time'
        db.add_column(u'confirmaction_action', 'live_time',
                      self.gf('django.db.models.fields.IntegerField')(null=True, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Action.executed'
        db.delete_column(u'confirmaction_action', 'executed')

        # Deleting field 'Action.action_status'
        db.delete_column(u'confirmaction_action', 'action_status')

        # Deleting field 'Action.result'
        db.delete_column(u'confirmaction_action', 'result')

        # Deleting field 'Action.live_time'
        db.delete_column(u'confirmaction_action', 'live_time')


    models = {
        u'confirmaction.action': {
            'Meta': {'object_name': 'Action'},
            'action_func': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'action_status': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'confirm_code': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'created': ('django.db.models.fields.DateTimeField', [], {}),
            'executed': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'live_time': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'result': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        u'confirmaction.actionarg': {
            'Meta': {'object_name': 'ActionArg'},
            'action': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'arg_set'", 'to': u"orm['confirmaction.Action']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        }
    }

    complete_apps = ['confirmaction']


# -*- coding: utf-8 -*-
from django.core.urlresolvers import reverse
from django.contrib.auth import get_user_model

from rest_framework import status
from rest_framework.test import APITestCase

from .. import exceptions
from ..main import set_action
from ..decorators import confirm_action


User = get_user_model()

@confirm_action
def activate_user_phone(user_pk):
    user = User.objects.get(pk=user_pk)
    user.is_phone_active = True
    user.save()
    return {'message': 'Phone active'}



@confirm_action
def system_fault():
    raise Exception


class ConfirmActionTestCase(APITestCase):

    def test_wrong_code(self):
        action = set_action(
            'zap@land.ru',
            'confirmaction.tests.test_view.activate_user_phone',
            {'user_pk': 12}
        )
        response = self.client.post(
            reverse('confirm-action', kwargs={'pk': action.pk}),
            {'code': '1481'}
        )
        self.assertEquals(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_function_fault(self):
        action = set_action(
            '+79625213997',
            'confirmaction.tests.test_view.system_fault',
            generate_code_func=lambda: '2048'
        )
        response = self.client.post(
            reverse('confirm-action', kwargs={'pk': action.pk}),
            {'code': '2048'},
        )
        self.assertEquals(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_all_ok(self):
        user = User.objects.create(
            name='test',
            phone='+79625213997',
            password='test'
        )
        action = set_action(
            user.phone,
            'confirmaction.tests.test_view.activate_user_phone',
            {'user_pk': user.pk},
            generate_code_func=lambda: '2048'
        )

        response = self.client.post(
            reverse('confirm-action', kwargs={'pk': action.pk}),
            {'code': '2048'}
        )
        self.assertEquals(response.status_code, status.HTTP_200_OK)

        user = User.objects.get(pk=user.pk)
        self.assertTrue(user.is_phone_active)



# -*- coding: utf-8 -*-
import datetime

import mock

from django import test

from .. import models
from .. import main
from .. import exceptions
from .. import settings
from ..decorators import confirm_action


@confirm_action
def simple_action(param_first, param_second):
    return {'status': 'done'}


@confirm_action
def error_during_process():
    raise exceptions.ErrorDuringProcess


@confirm_action
def system_fault():
    raise Exception

@confirm_action
def wrong_data_return():
    return 42


class SetActionTestCase(test.TestCase):

    def test_wrong_path(self):
        with self.assertRaises(exceptions.NotActionException):
            main.set_action('+7911', 'asdfsadf', {})

    def test_wrong_module(self):
        with self.assertRaises(exceptions.NotActionException):
            main.set_action('+7911', 'asdf.asdfsadf', {})

    def test_wrong_func(self):
        with self.assertRaises(exceptions.NotActionException):
            main.set_action('+7911', 'sys.asdfsadf', {})

    def test_wrong_function(self):
        with self.assertRaises(exceptions.NotActionException):
            main.set_action('+7911', 'inspect.isfunction', {})

    def test_all_ok(self):
        action = main.set_action(
            '+7911',
            'confirmaction.tests.test_main.simple_action',
            {
                'param_first': 'test',
                'param_second': 4
            }
        )
        self.assertIsNotNone(action.pk)

    def test_live_time(self):
        action = main.set_action(
            '+7911',
            'confirmaction.tests.test_main.simple_action',
            {
                'param_first': 'test',
                'param_second': 4
            },
            live_time=15
        )
        self.assertEquals(action.live_time, 15)

    def test_generation_func(self):
        gen_func = lambda: '2048'

        action = main.set_action(
            '+7911',
            'confirmaction.tests.test_main.simple_action',
            {
                'param_first': 'test',
                'param_second': 4
            },
            generate_code_func=gen_func
        )
        self.assertEquals(action.confirm_code, '2048')

    def test_send_func(self):
        class TestException(Exception):
            pass

        def send_func(user_contact, message):
            raise TestException

        with self.assertRaises(exceptions.DidNotSendMessage):
            main.set_action(
                '+7911',
                'confirmaction.tests.test_main.simple_action',
                {
                    'param_first': 'test',
                    'param_second': 4
                },
                send_code_func=send_func
            )


class ApplyActionTestCase(test.TestCase):

    def test_wrong_code(self):
        action = main.set_action(
            'zap@land.ru',
            'confirmaction.tests.test_main.simple_action',
            {
                'param_first': 'test',
                'param_second': 'param'
            }
        )

        with self.assertRaises(exceptions.WrongCode):
            main.apply_action(action.pk, '3482332')

    def test_used_code(self):
        gen_func = lambda: '2048'
        action = main.set_action(
            'zap@land.ru',
            'confirmaction.tests.test_main.simple_action',
            {
                'param_first': 'test',
                'param_second': 'param'
            },
            generate_code_func=gen_func
        )

        main.apply_action(action.pk, '2048')

        with self.assertRaises(exceptions.UsedAction):
            main.apply_action(action.pk, '2048')

    def test_out_of_date(self):
        gen_func = lambda: '2048'

        action = main.set_action(
            'zap@land.ru',
            'confirmaction.tests.test_main.simple_action',
            {
                'param_first': 'test',
                'param_second': 'param'
            },
            generate_code_func=gen_func,
            send_code_func=lambda *args: True
        )

        with mock.patch('confirmaction.models.Action.is_actual') as mock_meth:
            mock_meth.return_value = False
            with self.assertRaises(exceptions.OutOfDate):
                main.apply_action(action.pk, '2048')

    def test_action_done(self):
        gen_func = lambda: '2048'
        action = main.set_action(
            'zap@land.ru',
            'confirmaction.tests.test_main.simple_action',
            {
                'param_first': 'test',
                'param_second': 'param'
            },
            generate_code_func=gen_func
        )

        status, data = main.apply_action(action.pk, '2048')

        self.assertIn('status', data)

        self.assertEquals(status, models.Action.FINISHED_SUCCESS)

    def test_action_error_during_process(self):
        gen_func = lambda: '2048'
        action = main.set_action(
            'zap@land.ru',
            'confirmaction.tests.test_main.error_during_process',
            generate_code_func=gen_func
        )

        status, data = main.apply_action(action.pk, '2048')

        self.assertIn('error', data)
        self.assertEquals(status, models.Action.ERROR_DURING_PROCESS)

    def test_action_system_fault(self):
        gen_func = lambda: '2048'
        action = main.set_action(
            'zap@land.ru',
            'confirmaction.tests.test_main.system_fault',
            generate_code_func=gen_func
        )

        status, data = main.apply_action(action.pk, '2048')

        self.assertIn('error', data)
        self.assertEquals(status, models.Action.ACTION_FAULT)

    def test_action_wrong_data_returned(self):
        gen_func = lambda: '2048'
        action = main.set_action(
            '+8952342343',
            'confirmaction.tests.test_main.wrong_data_return',
            generate_code_func=gen_func
        )

        status, data = main.apply_action(action.pk, '2048')

        self.assertIn('error', data)
        self.assertEquals(status, models.Action.WRONG_RETURNED_DATA)

    def test_cant_find_action(self):
        main.set_action(
            'zap@land.ru',
            'confirmaction.tests.test_main.simple_action',
        )

        with self.assertRaises(exceptions.CantFindAction):
            main.apply_action(4096, '2048')

    def test_wrong_code_generator(self):
        original = settings.CONFIRM_GENERATION
        settings.CONFIRM_GENERATION = 'asdfasdf.asdf'
        with self.assertRaises(ValueError):
            main.set_action(
                'zap@land.ru',
                'confirmaction.tests.test_main.simple_action'
            )
        settings.CONFIRM_GENERATION = original

    def test_wrong_send_method(self):
        original = settings.CONFIRM_SEND_METHOD
        settings.CONFIRM_SEND_METHOD= 'asdfasdf.asdf'
        with self.assertRaises(ValueError):
            main.set_action(
                '+79625213997',
                'confirmaction.tests.test_main.simple_action'
            )
        settings.CONFIRM_SEND_METHOD = original




# -*- coding: utf-8 -*-
import os
from setuptools import setup

LONG_DESCRIPTION = """
Django confirmaction is a battery for confirm actions via email, sms etc.
"""


def long_description():
    try:
        return open(
            os.path.join(os.path.dirname(__file__), "README.md")
        ).read()
    except IOError:
        return LONG_DESCRIPTION

setup(
    name = "django-confirmaction",
    version = "0.0.2",
    author = "Aleksandr Aibulatov",
    author_email = "zap.aibulatov@gmail.com",
    description = "Django battery for confirm some action via email, sms, etc",
    license = "BSD",
    keywords = "django, battery, confirm action",
    url = "https://github.com/Zapix/django-confirmaction",
    packages=[
        'confirmaction',
        'confirmaction.tests',
        'confirmaction.migrations'
    ],
    long_description=long_description(),
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Framework :: Django",
        "Programming Language :: Python :: 2.7",
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License",
    ],
)


# -*- coding: utf-8 -*-
import inspect


def confirm_action(func):
    """
    Set
    """
    assert inspect.isfunction(func), "Func should be function"
    setattr(func, 'confirm_action', True)
    return func


from rest_framework import status
from rest_framework import response
from rest_framework.views import APIView

from . import exceptions
from .main import apply_action
from .models import Action


class ConfirmCodeView(APIView):

    def post(self, request, pk, *args, **kwargs):

        try:
            action_status, data = apply_action(pk,
                                               request.DATA.get('code', ''))
        except exceptions.CantFindAction as e:
            http_status = status.HTTP_400_BAD_REQUEST
            data = {'error': unicode(e)}
        except exceptions.UsedAction as e:
            http_status = status.HTTP_400_BAD_REQUEST
            data = {'error': unicode(e)}
        except exceptions.OutOfDate as e:
            http_status = status.HTTP_400_BAD_REQUEST
            data = {'error': unicode(e)}
        except exceptions.WrongCode as e:
            http_status = status.HTTP_400_BAD_REQUEST
            data = {'error': unicode(e)}
        else:
            if action_status == Action.FINISHED_SUCCESS:
                http_status = status.HTTP_200_OK
            else:
                http_status = status.HTTP_400_BAD_REQUEST

        return response.Response(data, status=http_status)



# -*- coding: utf-8 -*-
from django.conf import settings


CONFIRM_GENERATION = getattr(
    settings,
    'CONFIRM_GENERATION',
    'confirmaction.utils.generate_code'
)

CONFIRM_SEND_METHOD = getattr(
    settings,
    'CONFIRM_SEND_METHOD',
    'confirmaction.utils.send_code'
)

CODE_LIVE_TIME = getattr(
    settings,
    'CODE_LIVE_TIME',
    60
)

ACTIVATION_MESSAGE_TEMPLATE = getattr(
    settings,
    'ACTIVATION_MESSAGE_TEMPLATE',
    'confirmaction/activation_message.html'
)